"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeNewLibrary = void 0;
const fs_1 = require("fs");
const string_1 = require("../augments/string");
const library_migration_error_1 = require("../errors/library-migration-error");
const library_parse_error_1 = require("../errors/library-parse-error");
function makeNewLibrary({ oldLibrary, replacePaths, checkReplacementPaths = true, loggingEnabled = true, checkFiles = false, }) {
    const unreplacedPaths = new Set();
    const replacePathUsage = replacePaths.map(() => 0);
    const missingFiles = [];
    loggingEnabled && console.info(`Replacing locations...`);
    const newLibrary = {
        ...oldLibrary,
        Tracks: Object.keys(oldLibrary.Tracks).reduce((newTracks, trackKey) => {
            const oldTrack = oldLibrary.Tracks[trackKey];
            if (!oldTrack) {
                throw new library_parse_error_1.LibraryParseError(`Track key "${trackKey}" didn't actually exist in the library`);
            }
            const newTrack = {
                ...oldTrack,
            };
            const oldLocation = newTrack.Location;
            let markedForDeletion = false;
            if (oldLocation) {
                const replaced = replacePaths.some((replacePath, replacePathIndex) => {
                    let used = false;
                    if (oldLocation.includes(replacePath.old) ||
                        string_1.decodeLocation(oldLocation).includes(replacePath.old)) {
                        if ('delete' in replacePath && replacePath.delete) {
                            markedForDeletion = true;
                            used = true;
                        }
                        else if ('new' in replacePath && replacePath.new) {
                            newTrack.Location = oldLocation.replace(replacePath.old, replacePath.new);
                            if (newTrack.Location === oldLocation &&
                                !oldLocation.startsWith('http')) {
                                // try with sanitization
                                newTrack.Location = string_1.encodeLocation(string_1.decodeLocation(oldLocation).replace(replacePath.old, replacePath.new));
                            }
                            if (checkFiles &&
                                !newTrack.Location.startsWith('http') &&
                                !fs_1.existsSync(string_1.decodeLocation(newTrack.Location))) {
                                missingFiles.push({
                                    old: string_1.decodeLocation(oldLocation),
                                    new: string_1.decodeLocation(newTrack.Location),
                                });
                            }
                            used = true;
                        }
                        else {
                            throw new library_migration_error_1.LibraryMigrationError(`Replace path contained neither valid .new or valid .delete: ${JSON.stringify(replacePath)}`);
                        }
                    }
                    if (used) {
                        ++replacePathUsage[replacePathIndex];
                    }
                    return used;
                });
                if (!replaced) {
                    unreplacedPaths.add(oldLocation);
                }
            }
            if (!markedForDeletion) {
                newTracks[trackKey] = newTrack;
            }
            return newTracks;
        }, {}),
    };
    loggingEnabled && console.info(`Replacing finished`);
    const errors = [];
    if (checkReplacementPaths) {
        unreplacedPaths.forEach((path) => {
            errors.push(`This track location was not replaced:\n\t\t\t${path}`);
        });
        replacePathUsage.forEach((usage, index) => {
            if (!usage) {
                errors.push(`The following replacement was never used:\n\t\t\t${JSON.stringify(replacePaths[index], null, 4)}`);
            }
        });
    }
    if (missingFiles.length) {
        missingFiles.forEach((missingFile) => {
            errors.push(`\tMissing file:\n\t\t${JSON.stringify(missingFile, null, '\t\t\t').replace(/}$/, '\t\t}')}`);
        });
    }
    if (errors.length) {
        throw new library_migration_error_1.LibraryMigrationError('\n' + errors.join('\n'));
    }
    return newLibrary;
}
exports.makeNewLibrary = makeNewLibrary;
